<p class="logo">
  <a href="{{ pathto(master_doc) }}">
    <img class="logo" src="{{ pathto('_static/logo/wfuzz.svg', 1) }}"/>
  </a>
</p>
<p>

<p>
WFuzz is a web application security fuzzer tool and library for Python.
</p>


<h3>GitHub repository</h3>

<p>
Be part of the Wfuzz's community via <a href="https://github.com/xmendez/wfuzz/">GitHub tickets and pull requests</a>.</p> 

<iframe src="http://ghbtns.com/github-btn.html?user=xmendez&repo=wfuzz&type=watch&count=true&size=large"
  allowtransparency="true" frameborder="0" scrolling="0" width="200px" height="35px"></iframe>
</p>

<h3>Stay informed</h3>

<p>
Don’t forget to follow my github, twitter for news, releases and feedback.
</p>

<p><iframe src="http://ghbtns.com/github-btn.html?user=xmendez&type=follow&count=false"
  allowtransparency="true" frameborder="0" scrolling="0" width="200" height="20"></iframe></p>

<p>
<script async src="http://platform.twitter.com/widgets.js" charset="utf-8"></script>
<a href="https://twitter.com/x4vi_mendez" class="twitter-follow-button" data-show-count="false">Follow @x4vi_mendez</a>
</p>

